package ru.dbotthepony.mc.otm.registry

import net.minecraft.core.Registry
import net.minecraft.resources.ResourceKey
import net.neoforged.bus.api.IEventBus
import net.neoforged.neoforge.registries.NewRegistryEvent
import net.neoforged.neoforge.registries.RegistryBuilder
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.core.ResourceLocation
import kotlin.reflect.KProperty

object MBuiltInRegistries {
	private val delegates = ArrayList<Delegate<*>>()

	private class Delegate<T : Any>(private val key: ResourceKey<Registry<T>>, private val configurator: RegistryBuilder<T>.() -> Unit = {}) {
		init {
			delegates.add(this)
		}

		private var _value: Registry<T>? = null

		operator fun getValue(thisRef: Any, property: KProperty<*>): Registry<T> {
			return _value ?: throw IllegalStateException("Tried to access uninitialized registry ${key.location()}")
		}

		fun build(event: NewRegistryEvent) {
			if (_value != null) {
				throw IllegalStateException("Already created registry ${key.location()}!")
			}

			_value = RegistryBuilder(key).let {
				configurator.invoke(it)
				event.create(it)
			}
		}
	}

	val DECIMAL_PROVIDER_TYPE by Delegate(MRegistries.DECIMAL_PROVIDER_TYPE) {
		defaultKey(ResourceLocation(OverdriveThatMatters.MOD_ID, "zero"))
	}

	val MATTER_REGISTRY_ACTION by Delegate(MRegistries.MATTER_REGISTRY_ACTION)

	val MATTER_FUNCTION by Delegate(MRegistries.MATTER_FUNCTION) {
		defaultKey(ResourceLocation(OverdriveThatMatters.MOD_ID, "noop"))
	}

	val RECIPE_FINDER by Delegate(MRegistries.RECIPE_FINDER)
	val ANDROID_RESEARCH_DESCRIPTION by Delegate(MRegistries.ANDROID_RESEARCH_DESCRIPTION)
	val ANDROID_RESEARCH_RESULT by Delegate(MRegistries.ANDROID_RESEARCH_RESULT)
	val ANDROID_FEATURE by Delegate(MRegistries.ANDROID_FEATURE) { sync(true) }
	val STACK_TYPE by Delegate(MRegistries.STACK_TYPE)

	val ITEM_FILTER by Delegate(MRegistries.ITEM_FILTER)  {
		defaultKey(ResourceLocation(OverdriveThatMatters.MOD_ID, "empty"))
	}

	internal fun register(bus: IEventBus) {
		delegates.forEach { bus.addListener(it::build) }
	}
}
